--  2006 Peter Thiemann
{- | Database backend. Encapsulates DB specific stuff.
-}
module Database.TRANSDB.Backend where

-- import qualified Database.HDBC as DB
import qualified Database.TRANSDB.TraceQuery as DB

import qualified Database.TRANSDB.SQL as SQL


type PrimaryKey = String
type ColumnName = SQL.ColumnName
type TableName = SQL.TableName

-- | Create name of primary key column from table name.
primaryKeyColumn :: TableName -> ColumnName
primaryKeyColumn table =
  table ++ "_id"

-- | Create primary key sequence name from table name.
keySequenceName :: TableName -> TableName
keySequenceName table =
  table ++ '_' : primaryKeyColumn table ++ "_seq"

-- |generate a fresh primary key (a surrogate key)
generatePrimaryKey :: DB.Connection -> TableName -> IO PrimaryKey
generatePrimaryKey conn table =
  DB.handleSqlError $
  do let seqname = keySequenceName table
	 nextval_sql = "SELECT nextval (?)"
     st <- DB.prepare conn nextval_sql
     DB.sExecute st [Just seqname]
     r <- DB.sFetchRow st
     DB.commit conn
     case r of
       Just [Just pk] ->
	 return pk
       _ ->
	 error "TRANSDB.Backend.new.generatePrimaryKey"
